(function() {

    var PLAYER_V = 300;
    var BULLET_MIN_V = 180,
        BULLET_MAX_V = 250;

    var game = new Phaser.Game(500, 500, Phaser.CANVAS, 'game', { preload: preload, create: create, update: update }, false, false);


    var bullets;
    var playerBullets;
    var hasShoot = false;

    var score = 0,
        txtScore;

    var timeLineAddScore = new TimelineMax();

    function preload() {
        game.load.image("ball", "assets/ball2.png");
        game.load.image("player", "assets/ball_p2.png");
    }

    function create() {
        //Phaser.Canvas.setSmoothingEnabled(game.context, false);

        var text = "0";
        var style = { font: "96px Helvetica", fill: "rgb(255, 255, 255)", align: "center" };


        txtScore = game.add.text(game.world.centerX, game.world.centerY, text, style);
        txtScore.anchor.setTo(0.5, 0.5);
        txtScore.alpha = 0.15;

        bullets = game.add.group(null, 'bullets');
        playerBullets = game.add.group(null, 'playerbullets');

        for (var i = 0; i < 20; i++) {
            var x = randomRangeF(32, 512 - 32),
                y = randomRangeF(32, 512 - 32),
                v = randomRangeF(BULLET_MIN_V, BULLET_MAX_V) * (Math.random() < 0.5 ? -1 : 1);

            var s = bullets.create(x, y, 'ball');
            s.body.setSize(8, 8, 4, 4);
            if (Math.random() < 0.5) s.body.velocity.x = v;
            else s.body.velocity.y = v;

            s.events.onOutOfBounds.add(bulletOOB, this);
        }

        timeLineAddScore.fromTo(txtScore, 0.5, { alpha: 0.25}, { alpha: 0.5});
        timeLineAddScore.to(txtScore, 0.5, { alpha: 0.25});

        timeLineAddScore.play();

        game.input.onDown.add(handleClick, this);

    }

    function update() {
        game.physics.collide(playerBullets, bullets, collisionCallback, null, this);
    }

    function collisionCallback(objA, objB) {
        var isObjABullet = (objA.group.name == 'bullets');

        var bullet = isObjABullet ? objA : objB;
        //var player = isObjABullet ? objB : objA;

        var sx = bullet.x,
            sy = bullet.y;

        objA.kill();
        objB.kill();

        createPlayerSplitBullets(sx, sy, PLAYER_V);
    }

    function handleClick() {
        if (hasShoot) return;
        hasShoot = true;

        createPlayerSplitBullets(game.input.x - 8, game.input.y - 8, PLAYER_V);
    }

    function createPlayerSplitBullets(x, y, v) {
        var s = playerBullets.create(x, y, 'player');
        s.body.velocity.y = -v;
        s.body.setSize(8, 8, 4, 4);
        s.events.onOutOfBounds.add(playerOOB, this);

        s = playerBullets.create(x, y, 'player');
        s.body.velocity.y = v;
        s.body.setSize(8, 8, 4, 4);
        s.events.onOutOfBounds.add(playerOOB, this);

        s = playerBullets.create(x, y, 'player');
        s.body.velocity.x = -v;
        s.body.setSize(8, 8, 4, 4);
        s.events.onOutOfBounds.add(playerOOB, this);

        s = playerBullets.create(x, y, 'player');
        s.body.velocity.x = v;
        s.body.setSize(8, 8, 4, 4);
        s.events.onOutOfBounds.add(playerOOB, this);

        addScore(4);
    }

    function addScore(amount) {
        score += amount;
        txtScore.setText("" + score);
        timeLineAddScore.play(0);
    }


    function bulletOOB(bullet) {
        if (bullet.body.velocity.x != 0) {
            bullet.x = bullet.body.velocity.x > 0 ? - (512 + 32) : (512 + 32);
        } else {
            bullet.y = bullet.body.velocity.y > 0 ? - (512 + 32) : (512 + 32);
        }
    }

    function playerOOB(bullet) {
        bullet.kill();
        checkWin();
    }

    function checkWin() {
        var isPlayerEmpty = playerBullets.length == 0,
            isBulletsEmpty = bullets.length == 0;

        if (!isPlayerEmpty) {
            return;
        }

        txtScore.style.font = "48px Helvetica";

        if (isPlayerEmpty && isBulletsEmpty) {
            txtScore.setText("You WIN!\n" + score);
            timeLineAddScore.play(0);
            return;
        }

        txtScore.style.font = "48px Helvetica";
        txtScore.setText("Your Score\n" + score);
        timeLineAddScore.play(0);
    }

    /**
     * Generates a random number between min and max.
     *
     * @param {number} min
     * @param {number} max
     * @returns {number}
     */
    function randomRangeF(min, max) {
        if (max < min) {
            var t = min;
            min = max;
            max = t;
        }

        return Math.random() * (max - min) + min;
    }

})();